/*--------------------------------------------------------------------------
 *    
 *                         -- C N R S -- 
 *         Laboratoire d'Automatique et d'Analyse des Systemes 
 *                 7 Avenue du colonel Roche   
 *                     31 077 Toulouse Cedex
 *
 *  Auteur           : Sara Fleury
 *  Groupe           : Robotique et Intelligence Artificielle
 *  Nom du fichier   : piloCmdTaskFunc.h
 *  Date de creation : 1994
 * 
 *  Description      : Declaration de la structure de trajectoire de PILO
 *                     et celle des vitesses. Macro d'acces.
 *                     ATTENTION: Variables globales: 
 *                          piloSmoothTrajId et piloSpeedRefStr
 *                     -> bibliotheque non reentrante.
 *
 *
 *  Copyright (C) 1994 LAAS/CNRS.
 *----------------------------------------------------------------------*/

#ifndef PILO_CMD_TASK_H
#define PILO_CMD_TASK_H
 
#include "piloConst.h"
#include "piloStruct.h"


/* -----  VARIABLES TO BE DECLARED IN THE MAIN PROGRAM  ----- */
/* >>>>>>>>>>>>>  SEE END OF THIS FILE  <<<<<<<<<<<<<<<<<<<<< */

#ifndef MIN
#define MIN(x,y) ((y)<(x)?(y):(x))
#endif
#ifndef MAX
#define MAX(x,y) ((y)>(x)?(y):(x))
#endif

/* ---------------------    CONTANTS  ------------------------------ */

/* Lignes brisees: reduction de la longueur attribuee a une clotho */
#define FACT_MINOR_L_CLOTH                2.5

/* Filtrage des traj type ligne brisee + Test fin d'execution. */
#define EPSILON_DIST                      0.001   
#define EPSILON_ANGLE                     0.001   
#define EPSILON_EPS                       0.005   

/* Reduction des accelerations sur segments et arc de cercles */
#define PILO_REDUC_ACCEL_LIN        2. 
#define PILO_REDUC_ACCEL_ANG        2. 

/*---------------------- PARAMETRES DES TRAJECTOIRES --------------------*
*
* Il y a une structure SMOOTH_TRAJ par mouvement elementaire.
* 5 types de mouvement elementaire:
* - typeMvt: SEGMENT, TURN, CLOTHO, ANTI_CLOTHO, GOTO_END.    
* 
* - x0, y0, theta0, ro0, c0 caracterisent la config, le rayon de courbure 
*   et la courbure en debut de mouvement.
*
* - Les vitesses initiales v0, w0 et maximales vMax, wMax ainsi que les 
*   accelerations accelLin, accelAng sont celles calculees pour la 
*   TRAJECTOIRE NOMINALE. 
*
* - k est le parametre geometrique de CLOTHO ou ANTI_CLOTHO:
*   clotho: k = forw sqrt(pi/kc), avec kc = c/s (= aAng/v^2 si v=cst)
*   anti-clotho: k = ka = ro/theta ( = aLin/w^2 si w=cst)
*
* - varDist et varAng sont les variations curviligne et angulaire
*
* - segNum indique l'element de la ligne brise initiale au quel correspond 
*   le mouvement. (Uniquement indicatif, non utilise).
*
* - GOTO_END marque la fin de toute traj. La structure comporte ainsi dans
*   les champs x0, y0 et theta0 la configuration en fin de trajectoire.
*   C'est la derniere consigne emise. v0, w0, vMax et wMax sont mis a zero.
*/

typedef struct {
  double x0;             /* Abscisse en debut de mouvement */
  double y0;             /* Ordonnee */
  double theta0;         /* Angle */
  double v0;             /* Vitesse lineaire */
  double w0;             /* Vitesse angulaire */
  double ro0;            /* Rayon de courbure */
  double c0;             /* Courbure */
  PILO_TYPE_MVT typeMvt; /* type de mouvement */
  int segNum;            /* Pour info: num segment ligne brisee */
  double k;              /* Constante caracteristique pour clotho et anti */
  double varAng;         /* Variation angulaire */
  double varLin;         /* Variation lineaire */
  double vMax;           /* Vitessse lin. max pour la traj nominale */
  double wMax;           /* Vitesse ang. max  pour la traj nominale */
  double accelLin;       /* Accel lin  pour la traj nominale */
  double accelAng;       /* Accel ang  pour la traj nominale */
  double timeMin;        /* Temps minimum d'execution */
} SMOOTH_TRAJ, *SMOOTH_TRAJ_ID;

typedef struct PILO_SPEED_REF_STR {
  double vMax;               /* Vitesse lineaire max */
  double wMax;               /* Vitesse angulaire max */
  double accelLin;           /* Accel lin max */
  double accelAng;           /* Accel ang max */
}PILO_SPEED_REF_STR;


/* 
 * Macros d'acces a SMOOTH_TRAJ_ID *piloSmoothTrajId 
 */

#define X_START(i)       (piloSmoothTrajId[(i)].x0)
#define Y_START(i)       (piloSmoothTrajId[(i)].y0)
#define THETA_START(i)   (piloSmoothTrajId[(i)].theta0)
#define V_START(i)       (piloSmoothTrajId[(i)].v0)
#define W_START(i)       (piloSmoothTrajId[(i)].w0)
#define RO_START(i)      (piloSmoothTrajId[(i)].ro0)
#define C_START(i)       (piloSmoothTrajId[(i)].c0)
#define V_MAX(i)         (piloSmoothTrajId[(i)].vMax)
#define W_MAX(i)         (piloSmoothTrajId[(i)].wMax)
#define TYPE_MVT(i)      (piloSmoothTrajId[(i)].typeMvt)
#define SEG_NUM(i)       (piloSmoothTrajId[(i)].segNum)
#define K_TRAJ(i)        (piloSmoothTrajId[(i)].k)
#define VAR_ANG(i)       (piloSmoothTrajId[(i)].varAng)
#define VAR_LIN(i)       (piloSmoothTrajId[(i)].varLin)
#define ACCEL_LIN(i)     (piloSmoothTrajId[(i)].accelLin)
#define ACCEL_ANG(i)     (piloSmoothTrajId[(i)].accelAng)
#define TIME_MIN(i)      (piloSmoothTrajId[(i)].timeMin)

/* Macros d'acces a PILO_SPEED_REF_STR */

#define VMAX_REF         (piloSpeedRefStr.vMax)
#define WMAX_REF         (piloSpeedRefStr.wMax)
#define ALIN_REF         (piloSpeedRefStr.accelLin)
#define AANG_REF         (piloSpeedRefStr.accelAng)

/* Macros d'acces a SMOOTH_LOCAL_STR */

#define DATA_NB              (piloTrajStrId->header.nbSegs)
#define DATA_TYPE            (piloTrajStrId->header.segType)

#define DATA_ABS_OR_REL      (piloTrajStrId->header.absOrRel)
#define DATA_X0              (piloTrajStrId->header.x0)
#define DATA_Y0              (piloTrajStrId->header.y0)
#define DATA_THETA0          (piloTrajStrId->header.theta0)
#define DATA_DEV_THETA       (piloTrajStrId->header.maxInitAngleDev)
#define DATA_DEV_L           (piloTrajStrId->header.maxInitLengthDev)

#define DATA_DL(i)           (piloTrajStrId->seg[(i)].dL)
#define DATA_DTH(i)          (piloTrajStrId->seg[(i)].dTheta)
#define DATA_EPS(i)          (piloTrajStrId->seg[(i)].e)


/* Macros d'acces a SMOOTH_GLOBAL_STR */


#define DATAG_NB              (piloAbsBLStr.header.nbSegs)
  
#define DATAG_ABS_OR_REL      (piloAbsBLStr.header.absOrRel)
#define DATAG_X0              (piloAbsBLStr.header.x0)
#define DATAG_Y0              (piloAbsBLStr.header.y0)
#define DATAG_THETA0          (piloAbsBLStr.header.theta0)
#define DATAG_DEV_THETA       (piloAbsBLStr.header.maxInitAngleDev)
#define DATAG_DEV_L           (piloAbsBLStr.header.maxInitLengthDev)

#define DATAG_FINAL_DTHETA    (piloAbsBLStr.finalDTheta)

#define DATAG_X(i)            (piloAbsBLStr.point[(i)].x)
#define DATAG_Y(i)            (piloAbsBLStr.point[(i)].y)
#define DATAG_EPS(i)          (piloAbsBLStr.point[(i)].e)
#define DATAG_FORW(i)         (piloAbsBLStr.point[(i)].forward)
#define DATAG_TRIGO(i)        (piloAbsBLStr.point[(i)].trigo)

/* Macros d'acces a ARCS_TRAJ */

#define DATARS_NB          DATA_NB    
  
#define DATARS_ABS_OR_REL      DATA_INIT_POS_FLAG
#define DATARS_X0              DATA_X0
#define DATARS_Y0              DATA_Y0
#define DATARS_THETA0          DATA_THETA0
#define DATARS_DEV_THETA       DATA_DEV_THETA
#define DATARS_DEV_L           DATA_DEV_L

#define DATARS_DTH(i)          DATA_DTH(i)
#define DATARS_DL(i)           DATA_DL(i)
#define DATARS_RMIN(i)         DATA_EPS(i)

/* -----  VARIABLES TO BE DECLARED IN THE MAIN PROGRAM  ----- */

/* to be declared AND allocated AND initialised */
extern PILO_TRAJECTORY *piloTrajStrId;   /* input traj to be smooth */

/* to be declared AND allocated */
extern SMOOTH_TRAJ *piloSmoothTrajId;  /* output smoothed traj */

/* to be declared AND initialised */
extern PILO_SPEED_REF_STR piloSpeedRefStr;  /* Max speed ref */

extern PILO_ABSOLUTE_BL piloAbsBLStr;  /* input absolute traj */


#endif
